#include "application.h"

#include <QDir>
#include <stdexcept>
#include "settings/settingsdialog.h"

XkorApplication::XkorApplication(int & c, char * * v) : QApplication(c, v)
{
	settings = new QSettings("thirdgeek.com", "xkoranate");
	
	setDefaultSettings(); // in case we haven’t run the application before
	refreshSearchPaths(); // set the search path for sport files
	
	mainWindow = new XkorMainWindow;
        mainWindow->setUnifiedTitleAndToolBarOnMac(true);
	
	cw = new XkorCentralWidget;
	connect(cw, SIGNAL(fileChanged(QString)), this, SLOT(setFileName(QString)));
	connect(cw, SIGNAL(fileEdited()), this, SLOT(setModified()));

	// set up actions
	newAction = new QAction(QIcon(":/icons/document-new"), XkorApplication::tr("New file…"), this);
	newAction->setShortcut(QKeySequence::New);
	connect(newAction, SIGNAL(triggered()), cw, SLOT(newFile()));

	openAction = new QAction(QIcon(":/icons/document-open"), XkorApplication::tr("Open file…"), this);
	openAction->setShortcut(QKeySequence::Open);
	connect(openAction, SIGNAL(triggered()), cw, SLOT(openFile()));
	
	saveAction = new QAction(QIcon(":/icons/document-save"), XkorApplication::tr("Save file…"), this);
	saveAction->setShortcut(QKeySequence::Save);
	connect(saveAction, SIGNAL(triggered()), cw, SLOT(saveFile()));
	
	saveAsAction = new QAction(QIcon(":/icons/document-save-as"), XkorApplication::tr("Save file as…"), this);
	saveAsAction->setShortcut(Qt::CTRL + Qt::SHIFT + Qt::Key_S);
	connect(saveAsAction, SIGNAL(triggered()), cw, SLOT(saveFileAs()));
	
	settingsAction = new QAction(QIcon(":/icons/configure"), tr("Preferences"), this);
	connect(settingsAction, SIGNAL(triggered()), this, SLOT(editSettings()));

	// toolbar
	toolBar = new QToolBar;
	toolBar->setMovable(false);
	toolBar->addAction(newAction);
	toolBar->addAction(openAction);
	toolBar->addAction(saveAction);
	toolBar->addAction(saveAsAction);
	toolBar->addAction(settingsAction);

	mainWindow->setCentralWidget(cw);
	mainWindow->addToolBar(toolBar);
	mainWindow->setWindowTitle(tr("Untitled[*] – xkoranate"));
	mainWindow->setWindowIcon(QIcon(":/icons/xkoranate"));
	
	mainWindow->show();
}

void XkorApplication::editSettings()
{
	XkorSettingsDialog sd(mainWindow);
	sd.setSportDirectory(settings->value("sportDirectory").toString());
	int rval = sd.exec();
	if(rval == QDialog::Accepted)
	{
		settings->setValue("sportDirectory", sd.getSportDirectory());
		refreshSearchPaths();
		//ee->updateSportList(); // update the list of sports in the event editor to reflect the potentially-new sport directory
	}
}

bool XkorApplication::notify(QObject * rec, QEvent * ev)
{
	try
	{
		return QApplication::notify(rec, ev);
	}
	catch(std::exception & e)
	{
                qFatal("%s", e.what());
		return false;
	}
	catch(...)
	{
		qFatal("Unknown exception!");
		abort();
	}
}

void XkorApplication::refreshSearchPaths()
{
	QDir::setSearchPaths("sports", QStringList(settings->value("sportDirectory").toString()));
}

void XkorApplication::setDefaultSettings()
{
	if(!settings->contains("sportDirectory"))
	{
		if(QDir(QDir::currentPath() + "/xkoranate.app").exists())
			settings->setValue("sportDirectory", QDir::currentPath() + "/xkoranate.app/Contents/Resources/sports/");
		else
			settings->setValue("sportDirectory", QDir::currentPath() + "/sports/");
	}
}

void XkorApplication::setFileName(QString filename)
{
	QString displayName = tr("Untitled");
	if(filename != "")
		displayName = QFileInfo(filename).fileName();
	mainWindow->setWindowTitle(displayName + tr("[*] – xkoranate"));
	setModified(false);
}

void XkorApplication::setModified(bool newModified)
{
	modified = newModified;
	mainWindow->setWindowModified(newModified);
}
