#include "multipleruncompetitionoptions.h"

XkorMultipleRunCompetitionOptions::XkorMultipleRunCompetitionOptions(QHash<QString, QVariant> opts, QWidget * parent) : XkorAbstractOptionsWidget(opts, parent)
{
	runs = new QSpinBox;
	runs->setValue(options.value("runs", 2).toInt());
	runs->setRange(2, 99);
	connect(runs, SIGNAL(valueChanged(int)), this, SLOT(setRuns(int)));

	sortResults = new QCheckBox(tr("Sort results"));
	if(options.value("sortResults", "true") == "true")
		sortResults->setCheckState(Qt::Checked);
	connect(sortResults, SIGNAL(stateChanged(int)), this, SLOT(setSortResults(int)));

	sortByBestResult = new QCheckBox(tr("Sort by best result when total result is tied"));
	if(options.value("sortByBestResult", "true") == "true")
		sortByBestResult->setCheckState(Qt::Checked);
	sortByBestResult->setEnabled(sortResults->checkState() == Qt::Checked);
	connect(sortByBestResult, SIGNAL(stateChanged(int)), this, SLOT(setSortByBestResult(int)));

	QFormLayout * layout = new QFormLayout(this);
	layout->addRow(tr("Number of runs"), runs);
	layout->addRow(sortResults);
	layout->addRow(sortByBestResult);
}

void XkorMultipleRunCompetitionOptions::setRuns(int value)
{
	options["runs"] = value;
	emit optionsChanged(options);
}

void XkorMultipleRunCompetitionOptions::setSortByBestResult(int value)
{
	if(value == Qt::Checked)
		options["sortByBestResult"] = "true";
	else
		options["sortByBestResult"] = "false";
	emit optionsChanged(options);
}

void XkorMultipleRunCompetitionOptions::setSortResults(int value)
{
	if(value == Qt::Checked)
	{
		options["sortResults"] = "true";
		sortByBestResult->setEnabled(true);
		setSortByBestResult(sortByBestResult->checkState());
	}
	else
	{
		options["sortResults"] = "false";
		sortByBestResult->setEnabled(false);
		options["sortByBestResult"] = "false";
	}
	emit optionsChanged(options);
}
