#include "competitions/options/roundrobincompetitionoptions.h"

#include <algorithm>

XkorRoundRobinCompetitionOptions::XkorRoundRobinCompetitionOptions(QHash<QString, QVariant> opts, QWidget * parent) : XkorAbstractOptionsWidget(opts, parent)
{
	// set up basicOptions
	int numberOfLegs = (options.contains("numberOfLegs") ? options.value("numberOfLegs").toInt() : 1);
	
	basicOptions = new XkorRoundRobinBasicOptionsWidget(numberOfLegs);
	connect(basicOptions, SIGNAL(numberOfLegsChanged(int)), this, SLOT(setNumberOfLegs(int)));
	
	// set up tableOptions
	int pointsForWin = (options.contains("pointsForWin") ? options.value("pointsForWin").toInt() : 3);
	int pointsForDraw = (options.contains("pointsForDraw") ? options.value("pointsForDraw").toInt() : 1);
	int pointsForLoss = (options.contains("pointsForLoss") ? options.value("pointsForLoss").toInt() : 0);
	
	std::vector<QString> sortCriteria;
	if(options.contains("sortCriteria"))
	{
		QList<QVariant> oldCriteria = options.value("sortCriteria").toList();
		for(QList<QVariant>::iterator i = oldCriteria.begin(); i != oldCriteria.end(); ++i)
			sortCriteria.push_back(i->toString());
	}
	else
		sortCriteria = XkorSortCriteriaWidget::defaultSortCriteria();
	
	std::vector<XkorTableColumn> tableColumns;
	if(options.contains("tableColumns"))
	{
		std::vector<QString> tableColumnNames;
		QList<QVariant> oldColumnNames = options.value("tableColumns").toList();
		for(QList<QVariant>::iterator i = oldColumnNames.begin(); i != oldColumnNames.end(); ++i)
			tableColumnNames.push_back(i->toString());

		std::vector<QString> tableColumnHeadings;
		QList<QVariant> oldColumnHeadings = options.value("tableHeadings").toList();
		for(QList<QVariant>::iterator i = oldColumnHeadings.begin(); i != oldColumnHeadings.end(); ++i)
			tableColumnHeadings.push_back(i->toString());

		std::vector<QString> tableColumnWidths;
		QList<QVariant> oldColumnWidths = options.value("tableWidths").toList();
		for(QList<QVariant>::iterator i = oldColumnWidths.begin(); i != oldColumnWidths.end(); ++i)
			tableColumnWidths.push_back(i->toString());
		
		for(unsigned int i = 0; i < tableColumnNames.size() && i < tableColumnHeadings.size() && i < tableColumnWidths.size(); ++i)
			tableColumns.push_back(XkorTableColumn(tableColumnNames[i], tableColumnHeadings[i], tableColumnWidths[i].toInt()));
	}
	else
		tableColumns = XkorTableColumnsWidget::defaultColumns();
	
	tableOptions = new XkorTableOptionsWidget(sortCriteria, tableColumns, pointsForWin, pointsForDraw, pointsForLoss);
	connect(tableOptions, SIGNAL(sortCriteriaChanged()), this, SLOT(setSortCriteria()));
	connect(tableOptions, SIGNAL(tableColumnsChanged()), this, SLOT(setTableColumns()));
	connect(tableOptions, SIGNAL(pointsForWinChanged(int)), this, SLOT(setPointsForWin(int)));
	connect(tableOptions, SIGNAL(pointsForDrawChanged(int)), this, SLOT(setPointsForDraw(int)));
	connect(tableOptions, SIGNAL(pointsForLossChanged(int)), this, SLOT(setPointsForLoss(int)));
	
	// set up the tab widget
	tw = new QTabWidget;
	tw->insertTab(0, basicOptions, tr("Basic"));
	tw->insertTab(1, tableOptions, tr("Tables"));
	
	QGridLayout * layout = new QGridLayout(this);
	layout->addWidget(tw);
}

void XkorRoundRobinCompetitionOptions::setNumberOfLegs(int value)
{
	options.insert("numberOfLegs", QString::number(value));
	emit optionsChanged(options);
}

void XkorRoundRobinCompetitionOptions::setPointsForDraw(int value)
{
	options.insert("pointsForDraw", QString::number(value));
	emit optionsChanged(options);
}

void XkorRoundRobinCompetitionOptions::setPointsForLoss(int value)
{
	options.insert("pointsForLoss", QString::number(value));
	emit optionsChanged(options);
}

void XkorRoundRobinCompetitionOptions::setPointsForWin(int value)
{
	options.insert("pointsForWin", QString::number(value));
	emit optionsChanged(options);
}

void XkorRoundRobinCompetitionOptions::setSortCriteria()
{
	QList<QVariant> sortCriteria;
	std::vector<QString> criteriaVector = tableOptions->sortCriteria();
	for(std::vector<QString>::iterator i = criteriaVector.begin(); i != criteriaVector.end(); ++i)
		sortCriteria.append(*i);
	options.insert("sortCriteria", sortCriteria);
	emit optionsChanged(options);
}

void XkorRoundRobinCompetitionOptions::setTableColumns()
{
	QList<QVariant> tableColumns, tableHeadings, tableWidths;
	std::vector<XkorTableColumn> columnsVector = tableOptions->tableColumns();
	for(std::vector<XkorTableColumn>::iterator i = columnsVector.begin(); i != columnsVector.end(); ++i)
	{
		tableColumns.append(i->columnType);
		tableHeadings.append(i->heading);
		tableWidths.append(i->width);
	}
	options.insert("tableColumns", tableColumns);
	options.insert("tableHeadings", tableHeadings);
	options.insert("tableWidths", tableWidths);
	emit optionsChanged(options);
}
