#include "competitions/options/tableoptionswidget.h"

#include <QGridLayout>

XkorTableOptionsWidget::XkorTableOptionsWidget(std::vector<QString> & sortCriteria, std::vector<XkorTableColumn> & tableColumns, int pointsForWin, int pointsForDraw, int pointsForLoss, QWidget * parent) : QWidget(parent)
{
	sc = new XkorSortCriteriaWidget;
	sc->setSortCriteria(sortCriteria);
	connect(sc, SIGNAL(listChanged()), this, SLOT(setSortCriteriaChanged()));
	
	tc = new XkorTableColumnsWidget;
	tc->setColumns(tableColumns);
	connect(tc, SIGNAL(listChanged()), this, SLOT(setTableColumnsChanged()));
	
	ptsWin = new QSpinBox;
	ptsWin->setValue(pointsForWin);
	connect(ptsWin, SIGNAL(valueChanged(int)), this, SLOT(setPointsForWinChanged(int)));
	
	ptsDraw = new QSpinBox;
	ptsDraw->setValue(pointsForDraw);
	connect(ptsDraw, SIGNAL(valueChanged(int)), this, SLOT(setPointsForDrawChanged(int)));
	
	ptsLoss = new QSpinBox;
	ptsLoss->setValue(pointsForLoss);
	connect(ptsLoss, SIGNAL(valueChanged(int)), this, SLOT(setPointsForLossChanged(int)));
	
	// layout
	QFormLayout * upperForm = new QFormLayout;
	upperForm->addRow(tr("Sort criteria:"), sc);
	upperForm->addRow(tr("Table columns:"), tc);
	upperForm->setMargin(0);
	upperForm->setSpacing(3);
	
	QFormLayout * winForm = new QFormLayout;
	winForm->addRow(tr("Points for win:"), ptsWin);
	QFormLayout * drawForm = new QFormLayout;
	drawForm->addRow(tr("Draw:"), ptsDraw);
	QFormLayout * lossForm = new QFormLayout;
	lossForm->addRow(tr("Loss:"), ptsLoss);
	
	QGridLayout * layout = new QGridLayout(this);
	layout->addLayout(upperForm, 0, 0, 1, 3);
	layout->addLayout(winForm, 1, 0);
	layout->addLayout(drawForm, 1, 1);
	layout->addLayout(lossForm, 1, 2);
	layout->setVerticalSpacing(3);
}

void XkorTableOptionsWidget::setPointsForDrawChanged(int value)
{
	emit pointsForDrawChanged(value);
}

void XkorTableOptionsWidget::setPointsForLossChanged(int value)
{
	emit pointsForLossChanged(value);
}

void XkorTableOptionsWidget::setPointsForWinChanged(int value)
{
	emit pointsForWinChanged(value);
}

void XkorTableOptionsWidget::setSortCriteriaChanged()
{
	emit sortCriteriaChanged();
}

void XkorTableOptionsWidget::setTableColumnsChanged()
{
	emit tableColumnsChanged();
}

std::vector<QString> XkorTableOptionsWidget::sortCriteria() const
{
	return sc->sortCriteria();
}

std::vector<XkorTableColumn> XkorTableOptionsWidget::tableColumns() const
{
	return tc->columns();
}
