#include "filelineedit.h"

#include <QAction>
#include <QFileDialog>

XkorFileLineEdit::XkorFileLineEdit(const QString & caption, const QString & directory)
{
	isDirectory = true;
	
	c = caption;
	d = directory;
	
	init();
}

XkorFileLineEdit::XkorFileLineEdit(const QString & caption, const QString & directory, const QString & filter)
{
	isDirectory = false;
	
	c = caption;
	d = directory;
	f = filter;
	
	init();
}

XkorFileLineEdit::~XkorFileLineEdit()
{
	delete layout;
}

void XkorFileLineEdit::init()
{
	le = new QLineEdit;
	QAction * browseAction = new QAction(QIcon(":/icons/document-open"), tr("Browse…"), this);
	connect(browseAction, SIGNAL(triggered()), this, SLOT(openBrowseDialog()));
	browse = new QToolButton;
	browse->setDefaultAction(browseAction);
	connect(le, SIGNAL(editingFinished()), this, SLOT(leEditingFinished()));
	connect(le, SIGNAL(textChanged(QString)), this, SLOT(setTextChanged(QString)));
	
	layout = new QHBoxLayout(this);
	layout->setContentsMargins(0, 0, 0, 0);
	layout->addWidget(le);
	layout->addWidget(browse);
}

void XkorFileLineEdit::leEditingFinished()
{
	emit editingFinished(le->text());
}

void XkorFileLineEdit::openBrowseDialog()
{
	QString filename;
	if(isDirectory)
		filename = QFileDialog::getExistingDirectory(this, c, d);
	else
		filename = QFileDialog::getOpenFileName(this, c, d, f);
	le->setText(filename);
	emit editingFinished(filename);
}

void XkorFileLineEdit::setText(const QString & newText)
{
	le->setText(newText);
}

QString XkorFileLineEdit::text() const
{
	return le->text();
}
