#ifndef H2HPARADIGM_H
#define H2HPARADIGM_H

#include "paradigms/sqisparadigm.h"
#include "paradigms/options/h2hparadigmoptions.h"

class XkorH2HParadigm : public XkorSQISParadigm
{
	public:
		XkorH2HParadigm() : XkorSQISParadigm()
		{
			requiredValues.clear();
			requiredValues << "score";
		}
		XkorH2HParadigm(XkorSport * sport, QHash<QString, QVariant> userOptions) : XkorSQISParadigm(sport, userOptions)
		{
			requiredValues.clear();
			requiredValues << "score";
		}

		virtual bool hasOptionsWidget() { return true; }

		virtual XkorAbstractAthleteWidget * newAthleteWidget()
		{
			return XkorAbstractParadigm::newAthleteWidget();
		}

		virtual XkorAbstractOptionsWidget * newOptionsWidget(QHash<QString, QVariant> paradigmOptions)
		{
			return new XkorH2HParadigmOptions(paradigmOptions);
		}

	protected:
		virtual QPair<XkorResult, XkorResult> generateETScore(XkorResult home, XkorResult away, QString str)
		{
			QString scoreType = (str.isEmpty() ? "score" : str);
			home.result[scoreType] = home.value(scoreType).toDouble() + static_cast<int>(s->h2hScore("etScore", home.athlete.rpSkill, away.athlete.rpSkill));
			away.result[scoreType] = away.value(scoreType).toDouble() + static_cast<int>(s->h2hScore("etScore", away.athlete.rpSkill, home.athlete.rpSkill));

			return qMakePair(home, away);
		}

		virtual QPair<XkorResult, XkorResult> generateFTScore(XkorAthlete home, XkorAthlete away)
		{
			XkorResult hRes, aRes;
			hRes.athlete = home;
			aRes.athlete = away;

			hRes.setScore(static_cast<int>(s->h2hScore("score", home.rpSkill, away.rpSkill)));
			aRes.setScore(static_cast<int>(s->h2hScore("score", away.rpSkill, home.rpSkill)));

			return qMakePair(hRes, aRes);
		}

		virtual QPair<XkorResult, XkorResult> generateGGScore(XkorResult home, XkorResult away, QString str)
		{
			double goldenGoalProb = opt.value("goldenGoalProb").toDouble();

			if(s->randUniform() < goldenGoalProb) // someone scored; who was it?
			{
				double otScore1 = s->randWeighted(home.athlete.rpSkill, away.athlete.rpSkill);
				double otScore2 = s->randWeighted(away.athlete.rpSkill, home.athlete.rpSkill);

				if(otScore1 > otScore2)
					home.result[str] = home.value(str).toDouble() + 1;
				else
					away.result[str] = away.value(str).toDouble() + 1;
			}
			return qMakePair(home, away);
		}
};

#endif // H2HPARADIGM_H
