#ifndef NSFSPARADIGM_H
#define NSFSPARADIGM_H

#include "paradigms/abstracth2hparadigm.h"
#include "paradigms/options/nsfsparadigmoptions.h"

class XkorNSFSParadigm : public XkorAbstractH2HParadigm
{
	public:
		XkorNSFSParadigm() : XkorAbstractH2HParadigm() {;;;}
		XkorNSFSParadigm(XkorSport * sport, QHash<QString, QVariant> userOptions) : XkorAbstractH2HParadigm(sport, userOptions) {;;;}

		virtual bool hasOptionsWidget() { return true; }

		virtual XkorAbstractOptionsWidget * newOptionsWidget(QHash<QString, QVariant> paradigmOptions)
		{
			return new XkorNSFSParadigmOptions(paradigmOptions);
		}

	protected:
		virtual int generateScore(double skill, double oppSkill, double style, double oppStyle, bool homeAdvantage = false, double attackMultiplier = 1)
		{
			// load constants
			double baseAttackCoeff = opt.value("baseAttackCoeff").toDouble();
			double rankDiffModifier = opt.value("rankDiffModifier").toDouble();
			double rankCoeff = opt.value("rankCoeff").toDouble();
			double rankScalar = opt.value("rankScalar").toDouble();
			double bast = opt.value("baseAttackSuccessThreshold").toDouble();
			int baseAttacksSuperior = opt.value("baseAttacksSuperior").toInt();
			int baseAttacksInferior = opt.value("baseAttacksInferior").toInt();
			double attackCoeffSuperior = opt.value("attackCoeffSuperior").toDouble();
			double attackCoeffInferior = opt.value("attackCoeffInferior").toDouble();

			double styleOffset, styleScalar, styleCoeff;
			if(userOpt.value("NSFSStyleMods") == "true")
			{
				styleOffset = opt.value("styleOffset").toDouble();
				styleScalar = opt.value("styleScalar").toDouble();
				styleCoeff = opt.value("styleCoeff").toDouble();
			}
			else
			{
				styleOffset = styleScalar = 0;
				styleCoeff = 1;
			}

			int attacks;
			if(skill > oppSkill)
				attacks = static_cast<int>((baseAttacksSuperior + ((skill - oppSkill) * attackCoeffSuperior)) * pow(style + oppStyle + styleOffset, styleScalar) * styleCoeff * attackMultiplier);
			else
				attacks = static_cast<int>((baseAttacksInferior + ((skill - oppSkill) * attackCoeffInferior)) * pow(style + oppStyle + styleOffset, styleScalar) * styleCoeff * attackMultiplier);

			// calculate P(goal) on any given attack
			double pGoal = 1 - (bast + rankDiffModifier * (pow(oppSkill * rankCoeff, rankScalar) - pow(skill * rankCoeff, rankScalar))) / (baseAttackCoeff + homeAdvantage);

			// calculate P(0 goals), P (1 goal), …
			QHash<int, double> pGoals;
			pGoals[0] = pow(1 - pGoal, attacks);
			for(int i = 1; i <= attacks; ++i)
				pGoals[i] = pGoals[i - 1] * (attacks - i + 1) * pGoal / (i * (1 - pGoal));

			// score
			double rand = s->randUniform();
			double acc = 0;
			for(int i = 0; i <= attacks; ++i)
			{
				acc += pGoals[i];
				if(rand < acc)
					return i;
			}
			return -1;
		}

};

#endif
