#include "paradigms/options/nsfsparadigmoptions.h"

XkorNSFSParadigmOptions::XkorNSFSParadigmOptions(QHash<QString, QVariant> opts, QWidget * parent) : XkorAbstractOptionsWidget(opts, parent)
{
	homeAdvantage = new QCheckBox(tr("Home advantage"));
	if(options.value("homeAdvantage") == "true")
		homeAdvantage->setCheckState(Qt::Checked);
	setHomeAdvantage(homeAdvantage->checkState());
	connect(homeAdvantage, SIGNAL(stateChanged(int)), this, SLOT(setHomeAdvantage(int)));

	showTLAs = new QCheckBox(tr("Show nation codes"));
	if(options.value("showTLAs", "true") == "true")
		showTLAs->setCheckState(Qt::Checked);
	setShowTLAs(showTLAs->checkState());
	connect(showTLAs, SIGNAL(stateChanged(int)), this, SLOT(setShowTLAs(int)));

	// style modifiers box
	QGroupBox * styleModsBox = new QGroupBox(tr("Style modifiers"));
	xkorStyleMods = new QRadioButton(tr("xkoranate-style modifiers"));
	nsfsStyleMods = new QRadioButton(tr("NSFS-style modifiers (unbalanced)"));
	noStyleMods = new QRadioButton(tr("Disable style modifiers"));
	if(options.value("styleMods") != "false")
		xkorStyleMods->setChecked(true);
	else if(options.value("NSFSStyleMods") == "true")
		nsfsStyleMods->setChecked(true);
	else
		noStyleMods->setChecked(true);
	styleMods = new QButtonGroup;
	styleMods->addButton(xkorStyleMods);
	styleMods->addButton(nsfsStyleMods);
	styleMods->addButton(noStyleMods);
	connect(styleMods, SIGNAL(buttonClicked(int)), this, SLOT(setStyleMods()));

	QFormLayout * styleModsForm = new QFormLayout;
	styleModsForm->addRow(xkorStyleMods);
	styleModsForm->addRow(nsfsStyleMods);
	styleModsForm->addRow(noStyleMods);
	styleModsBox->setLayout(styleModsForm);

	// layout
	layout = new QGridLayout(this);
	layout->addWidget(homeAdvantage, 0, 0);
	layout->addWidget(showTLAs, 1, 0);
	layout->addWidget(styleModsBox, 2, 0);
}

XkorNSFSParadigmOptions::~XkorNSFSParadigmOptions()
{
	delete homeAdvantage;
	delete showTLAs;
}

void XkorNSFSParadigmOptions::setHomeAdvantage(int x)
{
	if(x == Qt::Checked)
		options.insert("homeAdvantage", "true");
	else
		options.insert("homeAdvantage", "false");
	emit optionsChanged(options);
}

void XkorNSFSParadigmOptions::setShowTLAs(int x)
{
	if(x == Qt::Checked)
		options.insert("showTLAs", "true");
	else
		options.insert("showTLAs", "false");
	emit optionsChanged(options);
}

void XkorNSFSParadigmOptions::setStyleMods()
{
	QAbstractButton * checkedButton = styleMods->checkedButton();
	if(checkedButton == xkorStyleMods)
	{
		options.insert("styleMods", "true");
		options.insert("NSFSStyleMods", "false");
	}
	else if(checkedButton == nsfsStyleMods)
	{
		options.insert("styleMods", "false");
		options.insert("NSFSStyleMods", "true");
	}
	else
	{
		options.insert("styleMods", "false");
		options.insert("NSFSStyleMods", "false");
	}
	emit optionsChanged(options);
}
