#include "paradigms/options/pgsparadigmoptions.h"

XkorPGSParadigmOptions::XkorPGSParadigmOptions(QHash<QString, QVariant> opts, QWidget * parent) : XkorAbstractOptionsWidget(opts, parent)
{
	showTLAs = new QCheckBox(tr("Show nation codes"));
	if(options.value("showTLAs") == "false")
		showTLAs->setCheckState(Qt::Unchecked);
	else
		showTLAs->setCheckState(Qt::Checked);
	setShowTLAs(showTLAs->checkState());
	connect(showTLAs, SIGNAL(stateChanged(int)), this, SLOT(setShowTLAs(int)));

	maxScore = new QDoubleSpinBox;
	maxScore->setRange(0, 1.5);
	maxScore->setSingleStep(0.01);
	maxScore->setValue(options.value("maxScore", 1.5).toDouble());
	connect(maxScore, SIGNAL(valueChanged(double)), this, SLOT(setMaxScore(double)));

	nameWidth = new QSpinBox;
	nameWidth->setRange(5, 200);
	nameWidth->setSingleStep(5);
	if(options.contains("nameWidth"))
		nameWidth->setValue(options.value("nameWidth").toInt());
	else
		nameWidth->setValue(40);
	setNameWidth(nameWidth->value());
	connect(nameWidth, SIGNAL(valueChanged(int)), this, SLOT(setNameWidth(int)));

	resultWidth = new QSpinBox;
	resultWidth->setRange(2, 20);
	if(options.contains("resultWidth"))
		resultWidth->setValue(options.value("resultWidth").toInt());
	else
		resultWidth->setValue(10);
	setResultWidth(resultWidth->value());
	connect(resultWidth, SIGNAL(valueChanged(int)), this, SLOT(setResultWidth(int)));

	// layout
	formLayout = new QFormLayout(this);
	formLayout->addRow(showTLAs);
	formLayout->addRow(tr("Maximum score after first run"), maxScore);
	formLayout->addRow(tr("Name column width"), nameWidth);
	formLayout->addRow(tr("Result column widths"), resultWidth);
}

XkorPGSParadigmOptions::~XkorPGSParadigmOptions()
{
	delete showTLAs;
	delete nameWidth;
	delete resultWidth;
	delete maxScore;
}

void XkorPGSParadigmOptions::setMaxScore(double x)
{
	options.insert("maxScore", x);
	emit optionsChanged(options);
}

void XkorPGSParadigmOptions::setShowTLAs(int x)
{
	if(x == Qt::Checked)
		options.insert("showTLAs", "true");
	else
		options.insert("showTLAs", "false");
	emit optionsChanged(options);
}

void XkorPGSParadigmOptions::setNameWidth(int x)
{
	options.insert("nameWidth", x);
	emit optionsChanged(options);
}

void XkorPGSParadigmOptions::setResultWidth(int x)
{
	options.insert("resultWidth", x);
	emit optionsChanged(options);
}

