#ifndef SQISPARADIGM_H
#define SQISPARADIGM_H

#include "paradigms/abstracth2hparadigm.h"
#include "paradigms/options/sqisparadigmoptions.h"

class XkorSQISParadigm : public XkorAbstractH2HParadigm
{
	public:
		XkorSQISParadigm() : XkorAbstractH2HParadigm() {;;;}
		XkorSQISParadigm(XkorSport * sport, QHash<QString, QVariant> userOptions) : XkorAbstractH2HParadigm(sport, userOptions) {;;;}
		
		virtual bool hasOptionsWidget() { return true; }
		
		virtual XkorAbstractOptionsWidget * newOptionsWidget(QHash<QString, QVariant> paradigmOptions)
		{
			return new XkorSQISParadigmOptions(paradigmOptions);
		}
	
	protected:
		virtual int generateScore(double skill, double oppSkill, double, double, bool homeAdvantage = false, double attackMultiplier = 1)
		{
			int attacks = static_cast<int>(opt.value("attacks").toDouble() * attackMultiplier);

			double a = opt.value("constantA").toDouble();
			double b = opt.value("constantB").toDouble();
			double homeAdvValue = (homeAdvantage ? opt.value("homeAdvantage").toDouble() : 1);

			// calculate P(goal) on any given attack
			double pGoal = (a + (b - (skill == oppSkill ? 1 : min(skill, oppSkill) / max(skill, oppSkill)) * b) * (skill > oppSkill ? 1 : -1)) * homeAdvValue;
			
			// calculate P(0 goals), P (1 goal), …
			QHash<int, double> pGoals;
			pGoals[0] = pow(1 - pGoal, attacks);
			for(int i = 1; i <= attacks; ++i)
				pGoals[i] = pGoals[i - 1] * (attacks - i + 1) * pGoal / (i * (1 - pGoal));
			
			// score
			double rand = s->randUniform();
			double acc = 0;
			for(int i = 0; i <= attacks; ++i)
			{
				acc += pGoals[i];
				if(rand < acc)
					return i;
			}
			return -1;
		}
};

#endif
