#ifndef WRESTLINGPARADIGM_H
#define WRESTLINGPARADIGM_H

#include "paradigms/abstractparadigm.h"

class XkorWrestlingParadigm : public XkorAbstractParadigm
{
	public:
		XkorWrestlingParadigm() : XkorAbstractParadigm()
		{
			supportedCompetitions.insert("matches", true);
		}

		XkorWrestlingParadigm(XkorSport * sport, QHash<QString, QVariant> userOptions) : XkorAbstractParadigm(sport, userOptions)
		{
			supportedCompetitions.insert("matches", true);
		}

		virtual void breakTie(QList<XkorAthlete>)
		{
			qDebug() << "unexpected tie break request in wrestling paradigm";
		}

		virtual void scorinate(QList<XkorAthlete> athletes, QList<XkorResult> = QList<XkorResult>())
		{
			for(QStringList::iterator i = requiredValues.begin(); i != requiredValues.end(); ++i)
				if(!(s->hasValue(*i)))
				{
					qDebug() << "missing parameter" << *i << "in XkorSQISParadigm::output(XkorResults *)";
					out.append(qMakePair(QString(""), QString("Sport does not support this paradigm")));
					return;
				}

			// initialize results
			out.clear();
			res.clear();

			int acc = 0;
			for(QList<XkorAthlete>::iterator away = athletes.begin(); away != athletes.end(); ++away)
			{
				if(acc & 1) // if we’re on the second team in a match
				{
					QList<XkorAthlete>::iterator home = away; // home = away - 1
					--home;

					QPair<int, int> score = generateScore(home->rpSkill, away->rpSkill);

					QString result = formatName(home->name, home->nation) + " " + formatScore(score.first, score.second) + " " + formatName(away->name, away->nation);

					out.append(qMakePair(home->name, result));
					res.append(XkorResult(score.first, *home));
					res.append(XkorResult(score.second, *away));
				}
				++acc;
			}
		}

	protected:
		virtual QString formatScore(int score1, int score2)
		{
			return QString::number(score1) + "–" + QString::number(score2);
		}

		virtual QPair<int, int> generateScore(double homeSkill, double awaySkill)
		{
			QPair<int, int> rval = qMakePair(0, 0);

			QList<QPair<QPair<int, int>, double> > probArr;
			probArr.append(qMakePair(qMakePair(3, 1), opt.value("p3-1").toDouble()));
			probArr.append(qMakePair(qMakePair(3, 0), opt.value("p3-0").toDouble()));
			probArr.append(qMakePair(qMakePair(4, 1), opt.value("p4-1").toDouble()));
			probArr.append(qMakePair(qMakePair(4, 0), opt.value("p4-0").toDouble()));
			probArr.append(qMakePair(qMakePair(5, 0), opt.value("p5-0").toDouble()));

			double homeRand = s->randWeighted(homeSkill, awaySkill);
			double awayRand = s->randWeighted(awaySkill, homeSkill);

			double difference;
			if(homeRand > awayRand)
				difference = s->randWeighted(homeSkill, awaySkill);
			else
				difference = s->randWeighted(awaySkill, homeSkill);

			double acc = 0;
			for(QList<QPair<QPair<int, int>, double> >::iterator i = probArr.begin(); i != probArr.end(); ++i)
			{
				acc += i->second;
				if(difference < acc)
				{
					if(homeRand > awayRand)
						rval = i->first;
					else
						rval = qMakePair(i->first.second, i->first.first);
					break;
				}
			}

			return rval;
		}
};

#endif // WRESTLINGPARADIGM_H
