#include "rpeditor.h"

#include <QFile>
#include <QFileDialog>
#include <QFormLayout>
#include <QGroupBox>
#include <QMessageBox>
#include <QToolButton>
#include <QtDebug>
#include "rplist.h"

#include "rpeditor/rpbonuswidgets/olympicrpbonuswidget.h"
#include "rpeditor/rpbonuswidgets/wc36rpbonuswidget.h"

XkorRPEditor::XkorRPEditor(QWidget * parent) : QWidget(parent)
{
	m_data = 0;
	isLoading = false;

	m_name = new QLineEdit;

	// general settings form
	QFormLayout * generalForm = new QFormLayout;
	generalForm->addRow(tr("Competition name:"), m_name);

	m_bonusEffect = new QDoubleSpinBox;
	m_bonusEffect->setMinimum(0);
	m_bonusEffect->setMaximum(100);
	m_bonusEffect->setSingleStep(1);
	m_bonusEffect->setSuffix("%");
	m_bonusEffect->setDecimals(1);
	m_bonusEffect->setValue(15);
	connect(m_bonusEffect, SIGNAL(valueChanged(double)), this, SLOT(setDataChanged()));

	m_rpCalcType = new QComboBox;
	m_rpCalcType->addItem(tr("Olympic style"), "olympic");
	m_rpCalcType->addItem(tr("WC36-style"), "wc36");
	
	// tree widget
	m_rpBonus = new XkorOlympicRPBonusWidget;
	
	// form layout
	QFormLayout * rpForm = new QFormLayout;
	rpForm->addRow(tr("Effect of RP bonus:"), m_bonusEffect);
	rpForm->addRow(tr("RP bonus type:"), m_rpCalcType);
	
	// tree widget layout
	m_bonusLayout = new QHBoxLayout;
	m_bonusLayout->addWidget(m_rpBonus);

	// RP bonus group
	QVBoxLayout * rpLayout = new QVBoxLayout;
	rpLayout->addLayout(rpForm, 0);
	rpLayout->addLayout(m_bonusLayout, 1);

	QGroupBox * rpGroup = new QGroupBox(tr("RP bonus"));
	rpGroup->setLayout(rpLayout);
	
	// layout
	QVBoxLayout * layout = new QVBoxLayout(this);
	layout->addLayout(generalForm, 0);
	layout->addWidget(rpGroup, 1);
        layout->setMargin(0);

	connect(m_name, SIGNAL(textChanged(QString)), this, SLOT(setDataChanged()));
	connect(m_rpCalcType, SIGNAL(currentIndexChanged(int)), this, SLOT(setDataChanged()));
	connect(m_rpCalcType, SIGNAL(currentIndexChanged(int)), this, SLOT(updateRPBonusWidget()));
	connect(m_rpBonus, SIGNAL(listChanged()), this, SLOT(setDataChanged()));
}

void XkorRPEditor::setData(XkorRPList * data)
{
	isLoading = true; // prevent dataChanged from being emitted
	if(data)
	{
		updateData(); // we want to save any edits to the previous signup list (if any)

		m_name->setText(data->competitionName());
		m_bonusEffect->setValue(data->rpEffect());
		m_rpCalcType->setCurrentIndex(m_rpCalcType->findData(data->rpCalculationType()));
		m_rpBonus->setBonuses(data->bonuses());
		m_rpBonus->setOptions(data->rpOptions());
		m_rpBonus->setMaxBonus(data->maxBonus());
		m_rpBonus->setMinBonus(data->minBonus());
	}
	else
		qDebug() << "Null pointer in XkorRPEditor::setData(XkorRPList *)";
	m_data = data;
	isLoading = false; // allow dataChanged to be emitted if the user does stuff
}

void XkorRPEditor::updateData()
{
	if(m_data)
	{
		m_data->setCompetitionName(m_name->text());
		m_data->setRPEffect(m_bonusEffect->value());
		m_data->setRPCalculationType(m_rpCalcType->itemData(m_rpCalcType->currentIndex()).toString());
		m_data->setBonuses(m_rpBonus->bonuses());
		m_data->setRPOptions(m_rpBonus->options());
		m_data->setMaxBonus(m_rpBonus->maxBonus());
		m_data->setMinBonus(m_rpBonus->minBonus());
	}
}

void XkorRPEditor::updateRPBonusWidget()
{
	QString newRPCalcType = m_rpCalcType->itemData(m_rpCalcType->currentIndex()).toString();
	if(m_currentRPCalcType != newRPCalcType && !isLoading)
	{
		if(m_rpBonus->bonuses().size() > 0)
		{
			int r = QMessageBox::warning(this, tr("xkoranate"), tr("Changing the bonus formula will delete all bonuses. Continue?"), QMessageBox::Yes, QMessageBox::No | QMessageBox::Default | QMessageBox::Escape);
			if(r == QMessageBox::No)
			{
				m_rpCalcType->setCurrentIndex(m_rpCalcType->findData(m_currentRPCalcType));
				return;
			}
		}
		
		m_bonusLayout->removeWidget(m_rpBonus);
		delete m_rpBonus;
		if(newRPCalcType == "wc36")
			m_rpBonus = new XkorWC36RPBonusWidget;
		else
			m_rpBonus = new XkorOlympicRPBonusWidget;
		connect(m_rpBonus, SIGNAL(listChanged()), this, SLOT(setDataChanged()));
		m_bonusLayout->addWidget(m_rpBonus);
		m_currentRPCalcType = newRPCalcType;
	}
}
