#include "tablegenerator/tablecolumndelegate.h"

#include <QComboBox>
#include <QLineEdit>
#include <QSpinBox>

QWidget * XkorTableColumnDelegate::createEditor(QWidget * parent, const QStyleOptionViewItem &, const QModelIndex & index) const
{
	if(index.column() == 0) // column type
	{
		QComboBox * comboBox = new QComboBox(parent);
		comboBox->setFrame(false);
		comboBox->insertItems(0, columnNames);
		return comboBox;
	}
	else if(index.column() == 2) // width
	{
		QSpinBox * spinBox = new QSpinBox(parent);
		spinBox->setFrame(false);
		return spinBox;
	}
	else
	{
		QLineEdit * lineEdit = new QLineEdit(parent);
		lineEdit->setFrame(false);
		return lineEdit;
	}
}

void XkorTableColumnDelegate::setEditorData(QWidget * editor, const QModelIndex & index) const
{
	if(index.column() == 0) // column type
	{
		QComboBox * comboBox = qobject_cast<QComboBox *>(editor);
		comboBox->setCurrentIndex(comboBox->findText(index.model()->data(index, Qt::DisplayRole).toString()));
	}
	else if(index.column() == 2) // width
	{
		QSpinBox * spinBox = qobject_cast<QSpinBox *>(editor);
		spinBox->setValue(index.model()->data(index, Qt::DisplayRole).toInt());
	}
	else
	{
		QLineEdit * lineEdit = qobject_cast<QLineEdit *>(editor);
		lineEdit->setText(index.model()->data(index, Qt::DisplayRole).toString());
	}
}

void XkorTableColumnDelegate::setModelData(QWidget * editor, QAbstractItemModel * model, const QModelIndex & index) const
{
	if(index.column() == 0) // column type
	{
		QComboBox * comboBox = qobject_cast<QComboBox *>(editor);
		QString name = comboBox->currentText();
		int n = columnNames.indexOf(name);
		if(n >= 0 && n < columnTypes.size())
			model->setData(index, columnTypes[n], 1093);
		else
			model->setData(index, tr("<unknown column type %1>").arg(name), 1093);
		model->setData(index, name);
	}
	else if(index.column() == 2) // width
	{
		QSpinBox * spinBox = qobject_cast<QSpinBox *>(editor);
		int value = spinBox->value();
		model->setData(index, value);
	}
	else
	{
		QLineEdit * lineEdit = qobject_cast<QLineEdit *>(editor);
		QString text = lineEdit->text();
		model->setData(index, text);
	}
}
